# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import unittest

from crotal.models import ObjectManager


class Number(object):
    """
    A class for testing purpose.
    """
    def __init__(self, num):
        self.num = num

class TestObjectManager(unittest.TestCase):

    def setUp(self):
        self.objects = ObjectManager()

    def test_add(self):
        self.objects.add('one', 1)
        self.objects.add('two', 2)
        self.objects['three'] =  3
        self.assertItemsEqual([1, 2, 3], self.objects.all())
        self.assertEqual(3, self.objects['three'])

    def test_sort(self):
        self.objects.add('five', Number(5))
        self.objects.add('three', Number(3))
        self.objects.add('four', Number(4))
        self.objects.add('two', Number(2))
        self.objects.add('one', Number(1))
        self.objects.sort(key='num')
        self.assertEqual([1, 2, 3, 4, 5], [obj.num for obj in self.objects.all()])

    def test_remove(self):
        self.objects.add('one', Number(1))
        self.objects.add('two', Number(2))
        self.objects.remove('one')
        self.assertItemsEqual([2], [obj.num for obj in self.objects.all()])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

from crotal.models import Model
from crotal.models.fields import *


class Template(Model):
    PATH = ['themes/{theme}/public/']

    path = CharField()
    layout = CharField()
    content = TextField()

    @classmethod
    def parse_content(cls, file_path, content, config):
        attributes = {}
        attributes['content'] = content
        attributes['path'] = file_path
        cls.parse_attributes(file_path, config, attributes)
        return cls(**attributes)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import os
import re
import yaml
import cPickle

from crotal import logger
from crotal.models.fields import *

ENCODING = 'base64'


class ObjectManager(object):

    def __init__(self):
        self._keys = []
        self._objects = {}

    def __contains__(self, item):
        return item in self._keys

    def __getitem__(self, item):
        return self._objects[item]

    def __setitem__(self, key, value):
        if key not in self._objects:
            self._keys.append(key)
        self._objects[key] = value

    def add(self, key, value):
        if key in self._keys:
            self._objects[key] = value
        else:
            self._keys.append(key)
            self._objects[key] = value

    def get(self, key, default=None):
        return self._objects.get(key, default)

    def remove(self, key):
        if key in self._keys:
            self._keys.remove(key)
            del self._objects[key]

    def all(self):
        return [self._objects[key] for key in self._keys]

    def sort(self, key, reverse=False):
        if not callable(key):
            sort_key = lambda x: getattr(x, key)
        else:
            sort_key = key
        self._keys.sort(key=lambda x: sort_key(self._objects[x]), reverse=reverse)


class BaseModel(type):

    def __new__(cls, name, bases, attrs):
        return super(BaseModel, cls).__new__(cls, name, bases, attrs)

    def __init__(cls, name, bases, attrs):

        fields = {}
        for attr_name, attr_value in attrs.iteritems():
            if isinstance(attr_value, Field):
                fields[attr_name] = attr_value
        for attr_name, field in fields.iteritems():
            delattr(cls, attr_name)
        setattr(cls, '_fields', fields)
        setattr(cls, 'name', name)
        setattr(cls, 'objects', ObjectManager())
        if 'PATH' in attrs:
            setattr(cls, 'PATH', [path.replace('/', os.sep) for path in attrs['PATH']])
            del attrs['PATH']
        super(BaseModel, cls).__init__(name, bases, attrs)


class Model():
    __metaclass__ = BaseModel

    def __init__(self, **attributes):
        for attr, value in attributes.iteritems():
            setattr(self, attr, value)
        self.create()

    def create(self):
        pass

    def serialize(self):
        return cPickle.dumps(self)

    def to_db(self, absolute_path):
        return {
            'content': self.serialize().encode('base64'),
            'last_mod_time': os.path.getmtime(absolute_path),
        }

    @classmethod
    def load(cls, database, config, update_data):
        cls.config = config
        if isinstance(cls.PATH, list):
            cls.PATH = [path.format(**config.__dict__) for path in cls.PATH]
        elif isinstance(cls.PATH, str):
            cls.PATH = [cls.PATH]
        cls._table = database.get_table(cls.__name__)
        cls.file_list = cls.get_file_list(config.base_dir, cls.PATH)
        cls.classify_files()
        cls.load_main_items(config)
        cls.load_extra_items(config)

    @classmethod
    def classify_files(cls):
        db_file_list = cls._table.content
        cls.modified_file_list = list(set(cls.file_list) - set(db_file_list))
        cls.unmodified_file_list = list(set(db_file_list) - (set(db_file_list) - set(cls.file_list)))
        cls.removed_file_list = list(set(db_file_list) - set(cls.file_list))
        for file_path in cls.unmodified_file_list:
            absolute_file_path = os.path.join(cls.config.base_dir, file_path)
            last_mod_time = os.path.getmtime(absolute_file_path)
            last_mod_time_in_db = cls._table.get(file_path)['last_mod_time']
            if last_mod_time != last_mod_time_in_db:
                cls.modified_file_list.append(file_path)
                cls.unmodified_file_list.remove(file_path)

    @classmethod
    def load_main_items(cls, config):
        for file_path in cls.modified_file_list:
            obj = cls.load_single_file(file_path, config)
            if obj:
                cls.objects.add(file_path, obj)
            else:
                logger.error(message="Incorrect file format: {0}".format(file_path))

        for file_path in cls.unmodified_file_list:
            cls.objects.add(file_path, Model.from_db(file_path, cls.config, cls._table[file_path]))

        for file_path in cls.removed_file_list:
            cls.remove_single_file(file_path)

        for obj in cls.objects.all():
            absolute_path = os.path.join(config.base_dir, obj.path)
            cls._table[obj.path] = obj.to_db(absolute_path)

    @classmethod
    def load_extra_items(cls, config):
        pass

    @classmethod
    def load_single_file(cls, file_path, config):
        item = cls.from_file(file_path, config)
        return item

    @classmethod
    def from_db(cls, path, config, row):
        serial = row['content'].decode('base64')
        return cPickle.loads(str(serial))

    @classmethod
    def from_file(cls, file_path, config):
        with open(file_path, 'r') as f:
            content = f.read().decode('utf8')
        return cls.parse_content(file_path, content, config)

    @classmethod
    def from_text(cls, file_path, text, config):
        return cls.parse_content(file_path, text, config)

    @classmethod
    def parse_content(cls, file_path, content, config):
        header = re.compile(r'---[\s\S]*?---').findall(content)
        if header:
            try:
                attributes = yaml.load(header[0].replace('---', ''))
            except yaml.YAMLError, e:
                logger.error("Wrong format inside the header area: {0}".format(file_path))
                return None
            attributes['content'] = content.replace(header[0], '', 1)
            attributes['path'] = file_path
            cls.parse_attributes(file_path, config, attributes)
            return cls(**attributes)
        else:
            logger.error("Can not find header area: {0}".format(file_path))
            return None

    @classmethod
    def parse_attributes(cls, file_path, config, attributes):
        for attr, field in cls._fields.iteritems():
            if attr in attributes:
                attributes[attr] = field.parse(attributes[attr])
            else:
                for other_name in field.other_names:
                    if other_name in attributes:
                        attributes[attr] = field.parse(attributes[other_name])

    @staticmethod
    def get_file_list(base_dir, path_list):
        """
        This method returns the list of all the source files in the directory
        indicated. Notice that file started with '.' will not be
        included.
        """
        file_list = []
        for path in path_list:
            absolute_path = os.path.join(base_dir, path)
            for dir_, _, files in os.walk(absolute_path):
                for file_name in files:
                    absolute_file = os.path.join(dir_, file_name)
                    if not file_name.startswith('.'):
                        file_path = os.path.relpath(absolute_file, base_dir)
                        file_list.append(file_path)
        return file_list

    @classmethod
    def single_file_on_change(cls, file_path, event_type, config):
        if event_type in ('created', 'modified'):
            obj = cls.load_single_file(file_path, config)
            if obj:
                cls.objects.add(file_path, obj)
            else:
                logger.error(message="Incorrect file format: {0}".format(file_path))
        elif event_type == 'deleted':
            cls.remove_single_file(file_path)
        if callable(getattr(cls, 'load_extra_items', None)):
            cls.load_extra_items(config)

    @classmethod
    def remove_single_file(cls, file_path):
        if file_path in cls.objects:
            cls.objects.remove(file_path)
        if file_path in cls._table:
            del cls._table[file_path]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import os
from inspect import ismethod

from crotal import utils
from crotal.template_engine import TemplateEngine
from crotal.models import *


class Renderer(object):

    def __init__(self, config):
        self.config = config
        self.site_content = {}
        self.templates = Template.objects.all()
        self.static_files = []
        self.variables = {
            'posts': Post.objects.all(),
            'pages': Page.objects.all(),
            'archives': Archive.objects.all(),
            'tags': Tag.objects.all(),
            'categories': Category.objects.all(),
        }
        self.variables.update({
            'site': self.config,
            'sidebar': True,
            'max_page': 5,
        })
        self.template_engine = TemplateEngine(self.config, self.templates)

    @utils.memoize
    def _layout_file(self, filename):
        if not filename.endswith('.html'):
            filename += '.html'
        return os.path.normpath(os.path.join(self.config.templates_dir,
                                             '_layout/', filename))

    def _update_variables(self, new_variables):
        v = self.variables.copy()
        v.update(new_variables)
        return v

    def render_static(self):
        for static_file in Static.objects.all():
            source_path = os.path.join(self.config.base_dir, static_file.path)
            path = utils.generate_path(static_file.url,
                                       output_path=self.config.publish_dir,
                                       site_root=self.config.root_path
                                       )
            self.static_files.append(path)
            utils.copy_file(source_path, path)

    def render_template(self):
        for template in self.templates:
            rel_path = os.path.relpath(template.path, self.config.templates_dir)
            if not rel_path.startswith('_'):
                content = self.template_engine.render(template.path, self.variables)
                path = utils.generate_path(rel_path, output_path=self.config.publish_dir, site_root=self.config.root_path)
                self.site_content[path] = content

    def render_post(self):
        layout_file = self._layout_file('post.html')
        for post in Post.objects.all():
            content = self.template_engine.render(layout_file, self._update_variables({'post': post}))
            path = utils.generate_path(post.url, output_path=self.config.publish_dir, site_root=self.config.root_path)
            self.site_content[path] = content

    def render_page(self):
        for page in Page.objects.all():
            layout_file = self._layout_file(page.layout)
            content = self.template_engine.render(layout_file, self._update_variables({'page': page}))
            path = utils.generate_path(page.url, output_path=self.config.publish_dir, site_root=self.config.root_path)
            self.site_content[path] = content

    def render_category_pages(self):
        layout_file = self._layout_file('categories.html')
        for category in Category.objects.all():
            content = self.template_engine.render(
                layout_file,
                self._update_variables({'category': category, 'title': category.name})
            )
            path = utils.generate_path(category.url, output_path=self.config.publish_dir, site_root=self.config.root_path)
            self.site_content[path] = content

    def render_tag_pages(self):
        layout_file = self._layout_file('tags.html')
        for tag in Tag.objects.all():
            content = self.template_engine.render(
                layout_file,
                self._update_variables({'tag': tag, 'title': tag.name})
            )
            path = utils.generate_path(tag.url, output_path=self.config.publish_dir, site_root=self.config.root_path)
            self.site_content[path] = content

    def render_archive_pages(self):
        layout_file = self._layout_file('archives.html')
        for archive in Archive.objects.all():
            content = self.template_engine.render(
                layout_file,
                self._update_variables({'archive': archive})
            )
            path = os.path.join('archives',
                                '{:04d}'.format(archive.datetime.year),
                                '{:02d}'.format(archive.datetime.month),
                                'index.html'
                                )
            path = utils.generate_path(archive.url, output_path=self.config.publish_dir, site_root=self.config.root_path)
            self.site_content[path] = content

    def render_index_pages(self):
        """
        render pages like '/page/2/'
        """
        layout_file = self._layout_file('index.html')
        page_number = len(self.variables['posts']) / (self.config.paginate + 1)
        for i in range(page_number + 1):
            v = self._update_variables({
                'index_posts': self.variables['posts'][i * self.config.paginate: i * self.config.paginate + self.config.paginate],
                'current_page': i + 1,
                'max_page': page_number + 1
            })
            content = self.template_engine.render(layout_file, v)
            if not i:
                path = utils.generate_path('index.html', output_path=self.config.publish_dir, site_root=self.config.root_path)
            else:
                path = utils.generate_path("page/{0}/".format(str(i+1)), output_path=self.config.publish_dir, site_root=self.config.root_path)
            self.site_content[path] = content

    def run(self):
        for name in dir(self):
            if name.startswith('render'):
                function = getattr(self, name)
                if ismethod(function):
                    function()
        return self.site_content, self.static_files



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import io
import os
import json

from crotal import logger


class Database(object):
    """
    Database is the interface to the file 'db.json'.
    table includes 'posts', 'pages', 'templates', 'static', 'static'.
    """
    @classmethod
    def from_file(cls, path):
        try:
            raw_db = json.loads(open(path, 'r').read())
        except:
            raw_db = {}
        return cls(path, content=raw_db)

    def __init__(self, path, content=None):
        if content:
            self.raw_db = content
        else:
            self.raw_db = {}
        self._path = path
        self._tables = {}
        for field in self.raw_db:
            self._tables[field] = Table(field, content=self.raw_db[field])

    def __getitem__(self, table):
        return self.get_table(table)

    def get_table(self, table):
        if not table in self._tables:
            self._tables[table] = Table(table, content={})
            logger.info('New table "{0}" created.'.format(table))
            return self._tables[table]
        else:
            return self._tables[table]

    def get_item(self, table, filename):
        return self[table].get(filename, {'content': None})

    def set_item(self, table, filename, item_dict):
        self[table][filename] = item_dict

    def remove_item(self, table, filename):
        if filename in self.db[table]:
            del self[table][filename]
        else:
            logger.warning("Failed to remove from database: {0}, TYPE: {1}".format(filename, table))

    def dump(self):
        json_output = {}
        for table in self._tables:
            json_output[table] = self._tables[table].content
        json_string = json.dumps(json_output, ensure_ascii=False)
        return json_string

    def save(self):
        io.open(self._path, 'w+', encoding='utf8').write(self.dump())


class Table(object):

    def __init__(self, table_name, content=None):
        self._table_name = table_name
        self._mapping = {} if not content else content

    @property
    def content(self):
        return self._mapping

    def __repr__(self):
        return '<%s:%s Keys:%r>' % (
            self.__class__.__name__,
            self._table_name,
            self._mapping.keys()
        )

    def __getitem__(self, item):
        return self._mapping.get(item)

    def __contains__(self, key):
        return key in self._mapping

    def __setitem__(self, key, value):
        self._mapping[key] = value

    def __delitem__(self, key):
        """Remove an item from the cache dict.
        Raise a `KeyError` if it does not exist.
        """
        try:
            del self._mapping[key]
        except KeyError, e:
            raise KeyError

    def keys(self):
        return self._mapping.keys()

    def get(self, key, default=None):
        """Return an item from the cache dict or `default`"""
        try:
            return self[key]
        except KeyError:
            return default



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import unittest
from datetime import datetime

from crotal.models import Page

SAMPLE_PAGE = """
---
title: A Sample Post
date: 2010-01-01 10:00
url: /unit/test/
---

This is the content of a post for test.
"""

SAMPLE_PAGE_WITH_WRONG_FORMAT = """
---
title: A Sample Post
date:2010-01-01 10:00
url: /unit/test/
---

This is the content of a post for test.
"""

class FakeConfig(object):
    permalink = ":title"
    author = "John Doe"


class TestPost(unittest.TestCase):

    def setUp(self):
        self.config = FakeConfig()
        self.sample_path = 'source/posts/sample.md'
        self.page = Page.from_text(self.sample_path, SAMPLE_PAGE, self.config)

    def test_page_with_wrong_format(self):
        page = Page.from_text(self.sample_path, SAMPLE_PAGE_WITH_WRONG_FORMAT, self.config)
        self.assertIsNone(page)

    def test_page_url(self):
        self.assertEqual('/unit/test/', self.page.url)

    def test_date(self):
        self.assertEqual(self.page.date, datetime(2010, 1, 1, 10, 00))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import unittest
import datetime

from crotal.models import Post


SAMPLE_POST = """
---
title: A Sample Post
date: 2010-01-01 10:00
categories: Sample, Test
tags: Unit Test
slug: a-sample-post
---

This is the content of a post for test.

"""

SAMPLE_POST_WITH_WRONG_FORMAT = """
---
title: A Sample Post
date: 2010-01-01 10:00
categories:Sample, Test
tags: Unit Test
slug: a-sample-post
---

This is the content of a post for test.
"""


class FakeConfig(object):
    permalink = ":title"
    author = "John Doe"


class TestPost(unittest.TestCase):

    def setUp(self):
        self.config = FakeConfig()
        Post.config = self.config
        self.sample_path = 'source/posts/sample.md'
        self.post = Post.from_text(self.sample_path, SAMPLE_POST, self.config)

    def test_post_with_wrong_format(self):
        post = Post.from_text(self.sample_path, SAMPLE_POST_WITH_WRONG_FORMAT, self.config,)
        self.assertIsNone(post)

    def test_post_categories(self):
        self.assertEqual(self.post.raw_categories, ['Sample', 'Test'])

    def test_post_tags(self):
        self.assertEqual(self.post.raw_tags, ['Unit Test'])

    def test_date(self):
        self.assertEqual(self.post.date, datetime.datetime(2010, 1, 1, 10, 00))

    def test_url(self):
        self.assertEqual(self.post.url, 'a-sample-post/')

    def test_url_with_different_permalink(self):
        self.config.permalink = "blog/:category/:year/:month/:day/:title"
        post = Post.from_text(self.sample_path, SAMPLE_POST, self.config)
        self.assertEqual(post.url, "blog/sample/2010/01/01/a-sample-post/")




import os

from crotal import utils
from crotal.site import Site

class Deployer(object):

    def __init__(self, config):
        self.config = config
        self.deploy_method = config.deploy_default

    def deploy(self):
        if (self.deploy_method == "git"):
            self.git_deploy()
        if (self.deploy_method == "rsync"):
            self.rsync()

    def git_deploy(self):
        git_remote = self.config.git_repository
        deploy_dir = os.path.join(self.config.base_dir, '.deploy')
        if not os.path.exists(deploy_dir):
            os.mkdir(deploy_dir)
            os.system("git clone {0} {1}".format(git_remote, deploy_dir))
        os.chdir(os.path.join(self.config.base_dir, '.deploy'))
        os.system("git pull")
        site = Site(path=utils.locate_base_dir(), full=True, output=deploy_dir)
        site.generate()
        os.system("git add --all *")
        os.system("git commit -m {0}".format("\"Crotal commit.\""))
        os.system("git push origin master")

    def rsync(self):
        deploy_dir = os.path.join(self.config.base_dir, 'deploy')
        site = Site(path=utils.locate_base_dir(), output=deploy_dir)
        site.generate()
        os.chdir(os.path.join(self.config.base_dir, 'deploy'))
        os.system("rsync -avz {0}\/* {1}:{2}".format(self.config.publish_dir,
                                                  self.config.ip,
                                                  self.config.remote_dir))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import os
try:
   from hashlib import md5
except ImportError:
   from md5 import md5
import imp

from crotal import db
from crotal import utils
from crotal import logger
from crotal.config import Config
from crotal.models import Page, Post, Template, Static


class Site(object):

    def __init__(self, path=os.getcwd(), full=False, output='preview'):
        self.config = Config(path, output)
        if full:
            self.database = db.Database(self.config.db_path)
        else:
            self.database = db.Database.from_file(self.config.db_path)
        self.data = {}
        self.site_content = {}
        self.static_files = []
        self.models = []
        self.load_modules()
        for model in [Page, Post, Template, Static]:
            model.load(self.database, self.config, self.update_data)
            self.models.append(model)

    def load_modules(self):
        renderer_path = os.path.join(self.config.base_dir, 'modules', 'renderer.py')
        if os.path.exists(renderer_path):
            try:
                self.Renderer = imp.load_source('renderer', renderer_path).Renderer
            except IOError:
                logger.error("Can not import {0}".format(renderer_path))
                from crotal import renderer
                self.Renderer = renderer.Renderer
        else:
            from crotal import renderer
            self.Renderer = renderer.Renderer

    def update_data(self, data):
        self.data.update(data)

    def generate(self):
        self.render()
        self.write()
        self.save()

    def parse_single_file(self, file_path, event_type):
        """
        This method is used when single file change was detected during the server
        running.
        """
        file_path = os.path.relpath(file_path, self.config.base_dir)
        model = self.detect_file_type(file_path)
        model.single_file_on_change(file_path, event_type, self.config)
        self.generate()

    def detect_file_type(self, file_path):
        for model in self.models:
            for path in model.PATH:
                if file_path.startswith(path):
                    return model

    def render(self):
        renderer = self.Renderer(self.config)
        self.site_content, self.static_files = renderer.run()

    def write(self):
        for root, dirs, files in os.walk(self.config.publish_dir):
            root_dir = os.path.relpath(root, self.config.publish_dir)
            if root_dir.startswith('.') and root_dir != '.':
                continue
            for file in files:
                file_path = os.path.join(root, file)
                if not file_path in self.site_content and \
                    not file_path in self.static_files:
                    os.unlink(file_path)
            if not os.listdir(root):
                os.rmdir(root)

        digest_table = self.database.get_table('digest')
        for path in self.site_content:
            digest = md5(self.site_content[path]).hexdigest()
            output_path = os.path.join(self.config.publish_dir, path)
            utils.output_file(output_path, self.site_content[path])
            digest_table[path] = digest

    def save(self):
        self.database.save()




# -*- coding: utf-8 -*
from __future__ import unicode_literals, print_function

import os
import shutil
import unittest

from crotal.site import Site
from base import BaseTest


SITE_MAP = [
    u'preview/tags/computer science/index.html',
    u'preview/tags/python/index.html',
    u'preview/about/index.html',
    u'preview/question-of-life-universe-everything/index.html',
    u'preview/archives/index.html',
    u'preview/categories/math/index.html',
    u'preview/page/2/index.html',
    u'preview/categories/others/index.html',
    u'preview/csapp/index.html',
    u'preview/tags/jersey devil/index.html',
    u'preview/tags/csapp/index.html',
    u'preview/archives/2013/11/index.html',
    u'preview/archives/2014/11/index.html',
    u'preview/robots.txt',
    u'preview/tags/monty hall/index.html',
    u'preview/categories/people/index.html',
    u'preview/monty-hall-problem/index.html',
    u'preview/about/cinema/index.html',
    u'preview/tags/larry wall/index.html',
    u'preview/categories/tutorial/index.html',
    u'preview/archives/2014/10/index.html',
    u'preview/make-some-sketches/index.html',
    u'preview/archives/2015/10/index.html',
    u'preview/larry-wall/index.html',
    u'preview/categories/book/index.html',
    u'preview/tags/jersey legends/index.html',
    u'preview/index.html',
    u'preview/tags/book/index.html',
    u'preview/archives/2012/10/index.html',
    u'preview/python-in-a-nutshell/index.html',
    u'preview/categories/web/index.html',
    u'preview/jersey-devil/index.html',
    u'preview/archives/2013/10/index.html',
    u'preview/rss.xml',
]


class TestSite(BaseTest):

    def setUp(self):
        self.site_path = os.path.join(self.data_dir, 'test_site')
        self.output_path = os.path.join(self.site_path, 'preview')
        if not os.path.exists(self.output_path):
            os.makedirs(self.output_path)
        shutil.rmtree(self.output_path)

    def test_site_generate_full(self):
        self.generate_site(full=True)
        file_list_after_generate = self.scan_files(self.output_path)
        self.assertSetEqual(set(SITE_MAP), set(file_list_after_generate))

    def test_site_generate(self):
        self.generate_site()
        file_list_after_generate = self.scan_files(self.output_path)
        self.assertSetEqual(set(SITE_MAP), set(file_list_after_generate))

    def generate_site(self, full=False):
        site = Site(self.site_path, full=full)
        site.generate()

    def scan_files(self, absolute_path):
        """
        This method returns the list of all the source files in the directory
        indicated. Notice that file started with '.' will not be
        included.
        """
        file_list = []
        for dir_, _, files in os.walk(absolute_path):
            for file_name in files:
                absolute_file = os.path.join(dir_, file_name)
                if not file_name.startswith('.'):
                    file_path = os.path.relpath(absolute_file, self.site_path)
                    file_list.append(file_path)
        return file_list

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function, unicode_literals

import os

from crotal.models import Model
from crotal.models.base import ObjectManager
from crotal.models.fields import *


class Static(Model):
    PATH = ['static/', 'themes/{theme}/static/']

    path = CharField()

    @classmethod
    def from_file(cls, file_path, config):
        return cls.parse_content(file_path, config)

    @classmethod
    def parse_content(cls, file_path, config):
        return cls(path = file_path)

    def __init__(self, path=""):
        self.path = path
        self.identify_output_path(self.PATH)

    def generate_url(self):
        url = []
        for item in self.output_path.split(os.path.sep):
            url.append(item)
        return "/".join(url)

    def identify_output_path(self, source_path):
        if isinstance(source_path, list):
            for path in source_path:
                if self.path.startswith(path):
                    self.output_path = os.path.relpath(self.path, path)
        else:
            self.output_path = os.path.relpath(self.path, source_path)
        self.url = self.generate_url()

class Category(object):
    URL_PATTERN = 'categories/:id'

    @classmethod
    def add(cls, name, obj):
        name = name.strip()
        slug = name.lower().replace(' ', '_')
        if hasattr(cls, 'objects'):
            object_manager = getattr(cls, 'objects')
            if slug in object_manager:
                object_manager[slug].insert(obj)
            else:
                object_manager[slug] = cls(name)
                object_manager[slug].insert(obj)
            return object_manager[slug]
        else:
            object_manager = ObjectManager()
            new_instance = cls(name)
            new_instance.insert(obj)
            object_manager.add(slug, new_instance)
            setattr(cls, 'objects', object_manager)
            return new_instance

    def __init__(self, name):
        self.name = name.strip()
        self.id = name.lower().replace(' ', '_')
        self.url = self.generate_url()
        self.posts = []

    def __eq__(self, other):
        return other.url == self.url

    def __repr__(self):
        return '<{0}: {1}>'.format(self.__class__.__name__, self.name)

    def __delitem__(self, key):
        for post in self.posts:
            if post.file_path == key:
                self.posts.remove(post)

    def generate_url(self):
        url = []
        for item in self.URL_PATTERN.split('/'):
            if item.startswith(':'):
                url.append(getattr(self, item[1:]).lower())
            else:
                url.append(item)
        return '/'.join(url)

    def insert(self, obj):
        if not filter(lambda x: x.path == obj.path, self.posts):
            self.posts.append(obj)

class Tag(object):
    URL_PATTERN = 'tags/:id'

    @classmethod
    def add(cls, name, obj):
        name = name.strip()
        slug = name.lower().replace(' ', '_')
        if hasattr(cls, 'objects'):
            object_manager = getattr(cls, 'objects')
            if slug in object_manager:
                object_manager[slug].insert(obj)
            else:
                object_manager[slug] = cls(name)
                object_manager[slug].insert(obj)
            return object_manager[slug]
        else:
            object_manager = ObjectManager()
            new_instance = cls(name)
            new_instance.insert(obj)
            object_manager.add(slug, new_instance)
            setattr(cls, 'objects', object_manager)
            return new_instance

    def __init__(self, name):
        self.name = name.strip()
        self.id = name.lower().replace(' ', '_')
        self.url = self.generate_url()
        self.posts = []

    def __eq__(self, other):
        return other.url == self.url

    def __repr__(self):
        return '<{0}: {1}>'.format(self.__class__.__name__, self.name)

    def __delitem__(self, key):
        for post in self.posts:
            if post.file_path == key:
                self.posts.remove(post)

    @classmethod
    def get_objects(cls):
        return cls.objects.values()

    def generate_url(self):
        url = []
        for item in self.URL_PATTERN.split('/'):
            if item.startswith(':'):
                url.append(getattr(self, item[1:]).lower())
            else:
                url.append(item)
        return '/'.join(url)

    def insert(self, obj):
        if not filter(lambda x: x.path == obj.path, self.posts):
            self.posts.append(obj)


class Archive(object):
    URL_PATTERN = 'archives/:year/:month'

    @classmethod
    def add(cls, datetime, obj):
        new_archive = cls(datetime)
        new_archive.insert(obj)
        if hasattr(cls, 'objects'):
            object_manager = getattr(cls, 'objects')
            if new_archive.__repr__() in object_manager:
                object_manager[new_archive.__repr__()].insert(obj)
            else:
                object_manager[new_archive.__repr__()] = new_archive
        else:
            object_manager = ObjectManager()
            object_manager.add(new_archive.__repr__(), new_archive)
            setattr(cls, 'objects', object_manager)

    def __init__(self, datetime):
        self.datetime = datetime
        self.year = datetime.strftime("%Y")
        self.month = datetime.strftime("%m")
        self.url = self.generate_url()
        self.posts = []

    def __repr__(self):
        return '{0}: {1}-{2}'.format(self.__class__.__name__, self.datetime.year, self.datetime.month)

    def __gt__(self, other):
        if self.datetime > other.datetime:
            return True
        else:
            return False

    def __lt__(self, other):
        if self.datetime < other.datetime:
            return True
        else:
            return False

    def __eq__(self, other):
        return other.datetime.year == self.datetime.year and other.datetime.month == self.datetime.month

    def insert(self, obj):
        if not filter(lambda x: x.path == obj.path, self.posts):
            self.posts.append(obj)

    def generate_url(self):
        url = []
        for item in self.URL_PATTERN.split('/'):
            if item.startswith(':'):
                url.append(getattr(self, item[1:]).lower())
            else:
                url.append(item)
        return '/'.join(url)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

from clint.textui import colored

def info(message):
    print("{0} {1}".format(colored.green('[info]'.format(info)),  message))

def warning(message):
    print("{0} {1}".format(colored.yellow('[warning]'.format(info)),  message))

def error(message):
    print("{0} {1}".format(colored.red('[error]'.format(info)),  message))

def success(message):
    print("{0} {1}".format(colored.blue('[success]'.format(info)),  message))

def yellow_text(info='', message=''):
    if info:
        print("{0} {1}".format(colored.yellow('[{0}]'.format(info)),  message))
    else:
        print(colored.yellow(message))

def red_text(info='', message=''):
    if info:
        print("{0} {1}".format(colored.red('[{0}]'.format(info)),  message))
    else:
        print(colored.red(message))

def blue_text(info='', message=''):
    if info:
        print("{0} {1}".format(colored.blue('[{0}]'.format(info)),  message))
    else:
        print(colored.blue(message))

def green_text(info='', message=''):
    colored_text = colored.green('[{0}]'.format(info))
    if info:
        print(colored.green('[{0}] '.format(info)) + message.encode('utf8'))
    else:
        print(colored.green(message))

def info_input(message):
    return raw_input(colored.green('[{0}]: '.format(message)))



from __future__ import unicode_literals, print_function

import os
import unittest

from crotal import utils


class TestUtils(unittest.TestCase):

    def setUp(self):
        pass

    def test_generate_path(self):
        path = utils.generate_path('/category/programming/', site_root='/demo/')
        self.assertEqual(path, os.path.join('demo', 'category', 'programming', 'index.html'))

    def test_generate_path_from_file(self):
        path = utils.generate_path('/category/programming/demo.html', site_root='/demo/')
        self.assertEqual(path, os.path.join('demo', 'category', 'programming', 'demo.html'))

    def test_generate_path_without_root(self):
        path = utils.generate_path('/category/programming/')
        self.assertEqual(path, os.path.join('category', 'programming', 'index.html'))



#!/usr/bin/python
# -*- coding:utf-8 -*-
"""Crotal - A static site generator
-----------------------------------

usage::

    crotal --help

"""
from __future__ import unicode_literals, print_function

import sys
import argparse

from crotal.command import Command
from crotal.version import __version__


DEFAULT_PORT = 1124

commands = {
    'generate': Command.generate,
    'init': Command.init_site,
    'server': Command.start_server,
    'new_post': Command.create_post,
    'new_page': Command.create_page,
    'deploy': Command.deploy,
}

def parse_arguments():
    parser = argparse.ArgumentParser(
        description="""A tool to generate static blogs,
        with markdown files.""",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    parser.add_argument('-v', '--version', action='version', version=__version__,
                        help='Print the Crotal version and exit.')

    subparsers = parser.add_subparsers(help='help')

    parser_init = subparsers.add_parser('init', help='Create & initiate a new site.')
    parser_init.add_argument('site_name', help='Name of the site you want to create.', type=str)

    parser_generate = subparsers.add_parser('generate', help='Generate the site, default in incremental mode.')
    parser_generate.add_argument('-f', '--full', action='store_true', help='Generate the full site, which takes a little longer.')
    parser_generate.add_argument('-o', '--output', help='Indicate the directory where you want to place the site generated.', type=str)

    parser_server = subparsers.add_parser('server', help='Start the Crotal server.')
    parser_server.add_argument('-p', '--port', help='Indicate another port you want to preview the site on.', default=DEFAULT_PORT, type=int)

    parser_new_post = subparsers.add_parser('new_post', help='Create a new post.')
    parser_new_post.add_argument('post_title', help='Title of the post you want to create.', type=str)

    parser_new_page = subparsers.add_parser('new_page', help='Create a new page.')

    parser_deploy = subparsers.add_parser('deploy', help='Deploy the site Crotal generated.')
    return parser.parse_args(sys.argv[1:])


def main():
    args = parse_arguments()
    main_command = sys.argv[1]
    func = commands[main_command]
    func(**vars(args))

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import os
import time
import zipfile
from datetime import datetime

from crotal import utils
from crotal import server
from crotal import logger
from crotal.site import Site
from crotal.config import Config
from crotal.deploy import Deployer
from crotal.version import __version__
from crotal.lib.pinyin.pinyin import PinYin


LOGO = \
"""  ____ ____   ___ _____  _    _
 / ___|  _ \\ / _ \\_   _|/ \\  | |
| |   | |_) | | | || | / _ \\ | |
| |___|  _ <| |_| || |/ ___ \\| |___
 \____|_| \_\\\\___/ |_/_/   \\_\\_____|
"""


LOGO_LARGE = (
        "________________________________________",
        "|   ____ ____   ___ _____  _    _      |",
        "|  / ___|  _ \\ / _ \\_   _|/ \\  | |     |",
        "| | |   | |_) | | | || | / _ \\ | |     |",
        "| | |___|  _ <| |_| || |/ ___ \\| |___  |",
        "|  \____|_| \_\\\\___/ |_/_/   \\_\\_____| |",
        "|                                      |",
        "|          Version: {:<6s}             |".format(__version__),
        "|______________________________________|",
        )

POST_SAMPLE =\
"""---
title: "{0}"
date: {1}
categories:
slug: {2}
---

"""

PAGE_SAMPLE = \
"""---
layout: page.html
title: "{0}"
date: {1}
url: {2}
description: {3}
---

## A demo page!

This is a demo page.
"""


class Command(object):

    @staticmethod
    def load_config(output=None):
        return Config(utils.locate_base_dir(), output)

    @staticmethod
    def generate(full=False, output=None):
        start = time.time()
        site = Site(path=utils.locate_base_dir(), full=full, output=output)
        site.generate()
        end = time.time()
        logger.info("Site generated in {0:.2f} seconds.".format(end-start))

    @staticmethod
    def init_site(site_name='crotal'):
        curr = os.path.dirname(os.path.abspath(__file__))
        sample_site = zipfile.ZipFile(os.path.join(curr, "init", "sample_site.zip"))
        if not os.path.exists(os.path.join(os.path.curdir, site_name)):
            os.mkdir(site_name)
        sample_site.extractall(path=os.path.join(os.path.curdir, site_name))

    @staticmethod
    def start_server(port):
        for line in LOGO_LARGE:
            logger.info(line)
        print()
        path = utils.locate_base_dir()
        server.start(port, path=path)

    @staticmethod
    def create_post(post_title='sample post'):
        config = Command.load_config()
        now = datetime.now()
        date = unicode(now.strftime('%Y-%m-%d %H:%M'))
        pinyin = PinYin()
        pinyin.load_word()
        slug = pinyin.hanzi2pinyin_split(string=post_title, split='-')
        new_post = POST_SAMPLE.format(post_title.decode('utf8'), date, slug)
        file_title = "{0}-{1}.markdown".format(unicode(now.strftime('%Y-%m-%d')), post_title.decode('utf8'))
        absolute_file_path = os.path.join(config.base_dir, (os.path.join(config.posts_dir, file_title)))
        if not os.path.exists(absolute_file_path):
            open(os.path.join(config.base_dir, absolute_file_path.encode('utf8')), 'w+').write(new_post.encode('utf8'))
        else:
            logger.error("File exists: {0}".format(absolute_file_path))

    @staticmethod
    def create_page():
        config = Command.load_config()
        title = logger.info_input('Page Title')
        url = logger.info_input('Page URL (.e.g, /foo/bar/):')
        description = logger.info_input('Page Description:')
        pinyin = PinYin()
        slug = pinyin.hanzi2pinyin_split(string=title, split="-")
        now = datetime.now()
        pub_date = now.strftime("%Y-%m-%d %H:%M")
        page = PAGE_SAMPLE.format(title, pub_date, url, description)
        file_path = os.path.join(config.pages_dir, "{0}.markdown".format(slug))
        open(os.path.join(config.base_dir, file_path), 'w+').write(page)
        logger.success('You can browse the page by {0} After generating the site.'.format(url))

    @staticmethod
    def deploy():
        config = Command.load_config()
        deployer = Deployer(config)
        deployer.deploy()



#!/usr/bin/env python
# -*- coding:utf-8 -*-

"""
    PinYin
    Author:cleverdeng
    E-mail:clverdeng@gmail.com
"""

__version__ = '0.9'
__all__ = ["PinYin"]

import os.path


class PinYin(object):

    def __init__(self):
        dir = os.path.dirname(os.path.abspath(__file__))
        self.word_dict = {}
        self.dict_file = dir + '/word.data'

    def load_word(self):
        if not os.path.exists(self.dict_file):
            raise IOError("NotFoundFile")

        with file(self.dict_file) as f_obj:
            for f_line in f_obj.readlines():
                try:
                    line = f_line.split('    ')
                    self.word_dict[line[0]] = line[1]
                except:
                    line = f_line.split('   ')
                    self.word_dict[line[0]] = line[1]

    def hanzi2pinyin(self, string=""):
        result = []
        if not isinstance(string, unicode):
            string = string.decode("utf-8")

        word = ''
        for char in string:
            key = '%X' % ord(char)
            if len(key) == 4:
                if word != '':
                    result.append(word)
                result.append(
                    self.word_dict.get(
                        key,
                        char).split()[0][
                        :-
                        1].lower())
                word = ''
            elif key == '20':
                if word != '':
                    result.append(word)
                word = ''
            else:
                word = word + char.encode('utf-8')
        if word != '':
            result.append(word)

        return result

    def hanzi2pinyin_split(self, string="", split=""):
        result = self.hanzi2pinyin(string=string)
        if split == "":
            return result
        else:
            return split.join(result)


if __name__ == "__main__":
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import os
import sys
import yaml

from crotal import logger


class Config(object):

    def __init__(self, path, output=None):
        if path is None or (not self.check_config(path)):
            logger.error("Config file '_config.yml' can not be loaded in the current directory and its parents.")
            sys.exit()
        self.base_dir = os.path.join(path)
        self.working_dir = os.getcwd()
        self.config_path = os.path.join(self.base_dir, '_config.yml')

        self.posts_dir = os.path.join('source', 'posts')
        self.pages_dir = os.path.join('source', 'pages')
        self.static_dir = os.path.join('static')

        self.db_path = os.path.join(self.base_dir, 'db.json')
        if output:
            self.publish_dir = os.path.join(self.base_dir, output)
        else:
            self.publish_dir = os.path.join(self.base_dir, 'preview')
        self.deploy_dir = os.path.join(self.base_dir, 'deploy')

        self.read_from_file()

    def __getattr__(self, item):
        return self.__dict__.get(item, None)

    def read_from_file(self):
        config_file = open(self.config_path, 'r').read()
        for item, value in yaml.load(config_file).iteritems():
            setattr(self, item, value)
        self.theme_dir = os.path.join('themes', self.theme)
        self.templates_dir = os.path.join(self.theme_dir, 'public')
        self.static_dir = os.path.join(self.theme_dir, 'static')

    def check_config(self, path):
        if not os.path.exists(os.path.join(path, '_config.yml')):
            return False
        else:
            return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

from datetime import datetime

import markdown
from markdown.extensions import Extension
from markdown.treeprocessors import Treeprocessor
from markdown.inlinepatterns import ImagePattern, IMAGE_LINK_RE

from crotal.models import Model
from crotal.models.others import Tag, Category, Archive
from crotal.models.fields import *


class CheckImagePattern(ImagePattern):

    def __init__(self, IMAGE_LINK_RE, md, config):
        self.config = config
        super(CheckImagePattern, self).__init__(IMAGE_LINK_RE, md)

    def handleMatch(self, m):
        node = ImagePattern.handleMatch(self, m)
        image_src = node.attrib['src']
        if not image_src.startswith('http'):
            if image_src.startswith('/'):
                node.attrib['src'] = "{0}{1}".format(self.config.root_path, image_src[1:])
        return node

FIELD_NAME_CONVERT = {
    'category': 'raw_categories',
    'categories': 'raw_categories',
    'tags': 'raw_tags',
    'tag': 'raw_tags',
    'date': 'date',
    'datetime': 'date'
}


# First create the treeprocessor

class ImgExtractor(Treeprocessor):

    def run(self, doc):
        "Find all images and append to markdown.images. "
        self.markdown.images = []
        for image in doc.findall('.//img'):
            self.markdown.images.append(image.get('src'))


class ImgExtExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        img_ext = ImgExtractor(md)
        md.treeprocessors.add('imgext', img_ext, '>inline')

MARKDOWN_EXTENSION_CONFIG = \
    {
        'codehilite': {},
    }


class Post(Model):
    """Model `Post` for posts in the blog.

    Attributes:
        PATH: The relative path where this model is related to, please
    use the relative path to this script.
        FILE_EXTENSIONS: Only the files with these file extensions shall
    be read by the model.

        title: Title of the post.
        slug: Slug of the post that may be used in its url.
        pub_date: Publication date of the post.
        tags: A list of Tags that the post is related to.
        categories: A list of Categories that the post belongs to.
        raw_content: The raw markdown content of the post.
        html_content: The html format post content generated from markdown
    file(raw_content).
        short_html_content: A short version of the `html_content`, which
    may be represented on the index page.

    """
    PATH = ['source/posts/']
    FILE_EXTENSIONS = ['.md', '.markdown']

    title = CharField(max_length=200)
    slug = CharField(max_length=200)
    date = DateTimeField(format="%Y-%m-%d %H:%M", other_names=['date'])
    raw_tags = ListField(content_type=str, other_names=['tag', 'tags'])
    raw_categories = ListField(content_type=str, other_names=['category', 'categories'])
    content = TextField()
    short_content = TextField()

    def create(self):
        md = markdown.Markdown(extensions=['fenced_code', 'codehilite', 'tables', ImgExtExtension()], extension_configs=MARKDOWN_EXTENSION_CONFIG)
        md.inlinePatterns['image_link'] = CheckImagePattern(IMAGE_LINK_RE, md, self.config)
        self.url = self.generate_url(self.config.permalink)
        self.content = md.convert(self.content)
        self.images = md.images
        self.short_content = self.content.split('<!--more-->')[0]
        self.categories = [Category.add(item, self) for item in self.raw_categories]
        self.tags = [Tag.add(item, self) for item in self.raw_tags]

    @classmethod
    def load_extra_items(cls, config):
        for object in cls.objects.all():
            Archive.add(object.date, object)
            for category in object.raw_categories:
                Category.add(category, object)
            for tag in object.raw_tags:
                Tag.add(tag, object)
        cls.objects.sort(key='date', reverse=True)
        Archive.objects.sort(key='datetime', reverse=True)
        Category.objects.sort(key=lambda x: len(x.posts), reverse=True)
        Tag.objects.sort(key=lambda x: len(x.posts), reverse=True)

    def generate_url(self, permalink):
        """
        Save the post url, refering to the permalink in the config file.
        The permalink is seperated into servel parts from '/'.
        If one part of it startswith ':', then we should find whether there
            is attribute with the same name in the post.
        If not, we regard it as a string
        example:
        post/:year/:month/:title
        will be generated to a url like 'crotal.org/post/2013/11/hello-world/'
        """
        url = ''
        for item in permalink.split('/'):
            if item.startswith(':'):
                url = url + \
                    self.escape_keywords(item.replace(':', '')) + '/'
            else:
                url = url + item + '/'
        return url.decode('utf8')

    def escape_keywords(self, word):
        return {
            'year': self.date.strftime('%Y'),
            'month': self.date.strftime('%m'),
            'day': self.date.strftime('%d'),
            'title': str(self.slug),
            'category': self.raw_categories[0].lower() if self.raw_categories else 'null',
        }[word]



# -*- coding: utf-8 -*
from __future__ import unicode_literals, print_function

import os
import unittest

from crotal.db import Table, Database
from base import BaseTest


class TestDatabase(BaseTest):

    def setUp(self):
        self.db_file_path = 'db.json'
        self.db = Database(self.db_file_path)

    def test_database_init(self):
        pass

    def test_table_set(self):
        self.db['table']['test_key'] = 'test_value'
        self.assertEqual(self.db.get_item('table', 'test_key'), 'test_value')

    def test_table_del(self):
        self.db['table']['test_key'] = 'test_value'
        del self.db['table']['test_key']
        self.assertIsNone(self.db.get_item('table', 'test_key'))

    def test_db_dump(self):
        self.db['table']['test_key'] = 'test_value'
        self.assertEqual(self.db.dump(), '{"table": {"test_key": "test_value"}}')

    def test_db_get_none(self):
        self.assertIsNone(self.db['table'].get('test_key_2'))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

from markdown import markdown

from crotal.models import Model
from crotal.models.fields import *


class Page(Model):
    """Model `Post` for posts in the blog.

    Attributes:
        PATH: The relative path where this model is related to, please
    use the relative path to this script.
        FILE_EXTENSIONS: Only the files with these file extensions shall
    be read by the model.

        title: Title of the post.
        slug: Slug of the post that may be used in its url.
        pub_date: Publication date of the post.
        tags: A list of Tags that the post is related to.
        categories: A list of Categories that the post belongs to.
        raw_content: The raw markdown content of the post.
        html_content: The html format post content genearted from markdown
    file(raw_content).
        short_html_content: A short version of the `html_content`, which
    may be represented on the index page.

    """
    PATH = ['source/pages/']
    FILE_EXTENSIONS = ['.md', '.markdown']

    title = CharField(max_length=200)
    slug = CharField(max_length=200)
    url = CharField()
    date = DateTimeField(format="%Y-%m-%d %H:%M")
    tags = ListField(content_type=str)
    categories = ListField(content_type=str)
    raw_content = TextField()
    html_content = TextField()
    short_html_content = TextField()

    def create(self):
        self.content = markdown(self.content, extensions=['fenced_code', 'codehilite', 'tables'])



from crotal.models.base import Model, ObjectManager
from crotal.models.page import Page
from crotal.models.post import Post
from crotal.models.template import Template
from crotal.models.others import Category, Archive, Tag, Static
from crotal.models.fields import *


import os

from mako.template import Template
from mako.lookup import TemplateLookup
from mako import exceptions

from crotal import logger


class Lookup(TemplateLookup):

    def adjust_uri(self, uri, relativeto):
        """
        Override `adjust_uri` so that we can locate template from absolute uri.
        """
        self.calling_uri = relativeto
        return uri

    def get_template(self, uri):
        """
        Override `get_template` so that it returns a `error template` when lookup
        failed to locate the template.
        """
        uri = uri.replace('/', os.sep)
        if uri in self._collection:
            return self._collection[uri]
        else:
            error_message = "template not found: {0}, calling by: {1}".format(uri, self.calling_uri)
            logger.error(error_message)
            error_template = Template(text=error_message)
            self.put_template(uri, Template(text=error_message))
            return error_template


class TemplateLoader(object):

    def __init__(self, config, mapping):
        """
        This is a wrap for the Lookup object.
        """
        self.mapping = mapping
        self.config = config
        self.lookup = Lookup(output_encoding='utf-8', filesystem_checks=False)
        for template in self.mapping:
            self.lookup.put_string(os.path.relpath(template.path, self.config.templates_dir), template.content)

    def get_template(self, file_path):
        return self.lookup.get_template(os.path.relpath(file_path, self.config.templates_dir))

    def list_templates(self):
        return sorted(self.mapping)


class TemplateEngine(object):
    """
    Engine uses Mako Template Engine to render template files.
    """
    def __init__(self, config, template_mapping):
        self.config = config
        self._template_mapping = template_mapping
        self._template_loader = TemplateLoader(config, self._template_mapping)

    def _layout_file(self, filename):
        if not filename.endswith('.html'):
            filename += '.html'
        return os.path.normpath(os.path.join(self.config.templates_dir,
                                             '_layout/', filename))

    def render(self, file_path, parameter):
        template = self._template_loader.get_template(file_path)
        try:
            return template.render(**parameter)
        except:
            logger.error("Template Error: {0}".format(file_path))
            return exceptions.html_error_template().render()



import os
import time
import shutil

from crotal import logger


def locate_base_dir():
    current_dir = os.getcwd()
    while True:
        if os.path.exists(os.path.join(current_dir, '_config.yml')):
            return current_dir
        elif current_dir == os.path.dirname(current_dir):
            return None
        else:
            current_dir = os.path.dirname(current_dir)


def make_dirs(file_path):
    dir_path = os.path.dirname(file_path)
    if not os.path.exists(dir_path):
        os.makedirs(dir_path)


def copy_file(src_dir, tar_dir):
    if os.path.exists(src_dir):
        make_dirs(tar_dir)
        shutil.copyfile(src_dir, tar_dir)


def copy_dir(src_dir, tar_dir):
    for item in os.listdir(src_dir):
        source_file = os.path.join(src_dir, item)
        target_file = os.path.join(tar_dir, item)

        if os.path.isfile(source_file):
            if not os.path.exists(tar_dir):
                os.makedirs(tar_dir)
            if not os.path.exists(tar_dir) or (os.path.exists(tar_dir) and (os.path.getsize(source_file))):
                open(target_file, "wb").write(open(source_file, "rb").read())
            else:
                pass
        if os.path.isdir(source_file):
            copy_dir(source_file, target_file)


def output_file(file_path, file_content):
    make_dirs(file_path)
    open(file_path, 'w+').write(file_content)


def generate_path(url, output_path='', site_root=''):
    """
    Generates a path based on url.
    :param output_path:
    Example:
        >>> generate_path('/category/programming/', output_path='/home/user/', site_root='/demo/')
        "/home/user/demo/category/programming/index.html"
    """
    path = [] if not site_root else [site_root.replace('/', '')]
    for item in url.split('/'):
        if item:
            path.append(item)
    if '.' not in path[-1] and path[-1].split('.'):
        path.append('index.html')
    return os.path.join(output_path, *path)


def stop_watch(func):
    def count_func_time(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        end = time.time()
        logger.info("Execution Time of {0}: {1:.2f} seconds".format(func.__name__, end - start))
        return result
    return count_func_time


def memoize(function):
    """
    Simply cache function returns.
    """
    memo = {}
    def wrapper(*args):
        if args in memo:
            return memo[args]
        else:
            rv = function(*args)
            memo[args] = rv
            return rv
    return wrapper


def get_subclasses(cls):
    return cls.__subclasses__() + [g for s in cls.__subclasses__()
                           for g in get_subclasses(s)]



#coding:utf-8
import os
import sys

from setuptools import setup, find_packages
from crotal import version


def read(fname):
    try:
        import pypandoc
        return pypandoc.convert(os.path.join(os.path.dirname(__file__), fname), 'rst')
    except (IOError, ImportError):
        return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "crotal",
    version = version.__version__,
    packages = find_packages(),

    include_package_data = True,

    entry_points = {
        'console_scripts' : [
            'crotal = crotal.main:main'
        ],
    },
    package_data = {
        'crotal' : [ '*.data' ]
    },
    install_requires = ['mako', 'markdown>=2.3.1', 'PyYAML>=3.10','Pygments>=1.6', 'clint>=0.4.1', 'watchdog>=0.8.1'],
    author = "Dinever",
    author_email = 'dingpeixuan911@gmail.com',
    url = "http://github.com/dinever/crotal",
    description = 'A static site framework written in Python. Fast, powerful, and easy as 1, 2, 3.',
    long_description = read('README.md'),
    license = "GNU GPLv3",
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 2 :: Only'
    ],
)



from datetime import datetime


class Field(object):
    pass


class CharField(Field):

    def __init__(self, max_length=None, other_names=None):
        self.max_length = max_length
        self.other_names = other_names if other_names else []

    def parse(self, input):
        if isinstance(input, basestring):
            return input[:self.max_length]
        else:
            return str(input)[:self.max_length]


class DateTimeField(Field):

    def __init__(self, format="", other_names=None):
        self.format = format
        self.other_names = other_names if other_names else []

    def parse(self, input):
        return datetime.strptime(input, self.format)


class ListField(Field):

    def __init__(self, content_type, other_names=None):
        self.content_type = content_type
        self.other_names = other_names if other_names else []

    def parse(self, input):
        """
        This method generates a list from a string with the format of "word1, word2, word3"
        """
        if input and (isinstance(input, str) or isinstance(input, unicode)):
            return [self.content_type(a.strip()) for a in input.split(',')]
        elif isinstance(input, list):
            return [self.content_type(a) for a in input]
        else:
            return list()


class TextField(Field):

    def __init__(self):
        self.other_names = []

    def parse(self, input):
        return input



import os
import sys
import time
import posixpath
import urllib
import BaseHTTPServer
from threading import Thread
from SimpleHTTPServer import SimpleHTTPRequestHandler

from watchdog.observers import Observer
from watchdog.events import PatternMatchingEventHandler

from crotal import logger
from crotal.site import Site
from crotal.config import Config
from crotal.version import __version__


LOGO = (
        "________________________________________",
        "|   ____ ____   ___ _____  _    _      |",
        "|  / ___|  _ \\ / _ \\_   _|/ \\  | |     |",
        "| | |   | |_) | | | || | / _ \\ | |     |",
        "| | |___|  _ <| |_| || |/ ___ \\| |___  |",
        "|  \____|_| \_\\\\___/ |_/_/   \\_\\_____| |",
        "|                                      |",
        "|          Version: {:<6s}             |".format(__version__),
        "|______________________________________|",
        )


class RequestHandler(SimpleHTTPRequestHandler):

    def log_request(self, code='-', size='-'):
        pass

    def log_message(self, format, *args):
        if args[0] == 404:
            logger.yellow_text('404', message='Can not get file: {0}'.format(self.path))

    def translate_path(self, path):

        root = os.getcwd()

        for pattern, root_dir in RequestHandler.routes:
            if path.startswith(pattern):
                path = path[len(pattern):]
                root = root_dir
                break

        path = path.split('?', 1)[0]
        path = path.split('#', 1)[0]
        path = posixpath.normpath(urllib.unquote(path))
        words = path.split('/')
        words = filter(None, words)

        path = root
        for word in words:
            drive, word = os.path.splitdrive(word)
            head, word = os.path.split(word)
            if word in (os.curdir, os.pardir):
                continue
            path = os.path.join(path, word)

        return path


class ServerThread(Thread):

    def __init__(self, config, port):
        self.config = config
        self.routes = (
            ['', config.publish_dir],
        )
        self.port = port
        self.help_info = (
            "Server started in `{0}`'".format(self.config.publish_dir),
            "To see your site, visit http://localhost:{0}".format(self.port),
            "To shut down Crotal, press <CTRL> + C at any time."
        )
        super(ServerThread, self).__init__()

    def run(self):
        del sys.argv[0]
        self._run(RequestHandler, BaseHTTPServer.HTTPServer)

    def _run(self, handler_class,
         server_class, protocol="HTTP/1.0"):
        """
        Test the HTTP request handler class.

        This runs an HTTP server on port 1124 (or the first command line
        argument).

        """

        server_address = ('', self.port)

        handler_class.protocol_version = protocol
        handler_class.routes = self.routes

        httpd = server_class(server_address, handler_class)

        for line in self.help_info:
            logger.info(line)
        print
        httpd.serve_forever()


class ChangeHandler(PatternMatchingEventHandler):
    patterns = ["*.*"]

    def __init__(self, config, site, ignore_patterns=None):
        self.config = config
        self.site = site
        super(ChangeHandler, self).__init__(ignore_patterns=ignore_patterns)

    def process(self, event):
        self.site.parse_single_file(event.src_path.decode('utf8'), event.event_type)

    def on_modified(self, event):
        logger.green_text('update', event.src_path.decode('utf8'))
        self.process(event)

    def on_created(self, event):
        logger.green_text('create', event.src_path.decode('utf8'))
        self.process(event)

    def on_deleted(self, event):
        logger.green_text('remove', event.src_path.decode('utf8'))
        self.process(event)


def start(port, path=os.getcwd()):
    if not path:
        logger.error("Not in a crotal site.")
        sys.exit()
    site = Site(path)
    site.generate()
    config = Config(path)
    serverThread = ServerThread(config, port)
    serverThread.daemon = True
    serverThread.start()
    observer = Observer()
    for model in site.models:
        for path in model.PATH:
            observer.schedule(ChangeHandler(config, site, ignore_patterns=["*/.DS_Store"]),
                              path=os.path.join(config.base_dir, path), recursive=True)
    observer.start()
    while True:
        try:
            time.sleep(1)
        except KeyboardInterrupt:
            logger.info('Server shutting down.')
            sys.exit()

